"use strict";Object.defineProperty(exports, "__esModule", {value: true});var _encoding = require('@cosmjs/encoding');








var _types = require('edge-core-js/types');





var _swapHelpers = require('../../util/swapHelpers');
var _utils = require('../../util/utils');


const swapInfo = {
  pluginId: 'cosmosibc',
  displayName: 'Cosmos IBC',
  orderUri: undefined,
  isDex: true,
  supportEmail: 'support@edge.com'
}

 function makeCosmosIbcPlugin(
  opts
) {
  const fetchSwapQuoteInner = async (
    request
  ) => {
    const {
      publicAddress: fromAddress
    } = await request.fromWallet.getReceiveAddress({
      tokenId: request.fromTokenId
    })
    const {
      publicAddress: toAddress
    } = await request.toWallet.getReceiveAddress({ tokenId: request.toTokenId })

    // Make sure both plugins are Cosmos-based
    try {
      _encoding.fromBech32.call(void 0, fromAddress)
      _encoding.fromBech32.call(void 0, toAddress)
    } catch (e) {
      throw new (0, _types.SwapCurrencyError)(swapInfo, request)
    }

    const spendInfo = {
      tokenId: request.fromTokenId,
      spendTargets: [
        {
          nativeAmount: request.nativeAmount,
          publicAddress: toAddress
        }
      ],
      assetAction: {
        assetActionType: 'transfer'
      },
      swapData: {
        isEstimate: false,
        payoutAddress: toAddress,
        plugin: { ...swapInfo },
        payoutCurrencyCode: request.toCurrencyCode,
        payoutNativeAmount: request.nativeAmount,
        payoutWalletId: request.toWallet.id
      },
      savedAction: {
        actionType: 'swap',
        swapInfo,
        isEstimate: false,
        canBePartial: false,
        fromAsset: {
          pluginId: request.fromWallet.currencyInfo.pluginId,
          tokenId: request.fromTokenId,
          nativeAmount: request.nativeAmount
        },
        toAsset: {
          pluginId: request.toWallet.currencyInfo.pluginId,
          tokenId: request.toTokenId,
          nativeAmount: request.nativeAmount
        },
        payoutAddress: toAddress,
        payoutWalletId: request.toWallet.id
      }
    }

    return {
      request,
      spendInfo,
      swapInfo,
      fromNativeAmount: request.nativeAmount
    }
  }

  const out = {
    swapInfo,

    async fetchSwapQuote(req) {
      const request = _utils.convertRequest.call(void 0, req)

      if (
        request.fromWallet.currencyInfo.pluginId ===
          request.toWallet.currencyInfo.pluginId ||
        request.fromCurrencyCode !== request.toCurrencyCode
      ) {
        throw new (0, _types.SwapCurrencyError)(swapInfo, request)
      }

      const newRequest = await _swapHelpers.getMaxSwappable.call(void 0, fetchSwapQuoteInner, request)
      const swapOrder = await fetchSwapQuoteInner(newRequest)
      return await _swapHelpers.makeSwapPluginQuote.call(void 0, swapOrder)
    }
  }

  return out
} exports.makeCosmosIbcPlugin = makeCosmosIbcPlugin;
