"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; } function _nullishCoalesce(lhs, rhsFn) { if (lhs != null) { return lhs; } else { return rhsFn(); } } function _optionalChain(ops) { let lastAccessLHS = undefined; let value = ops[0]; let i = 1; while (i < ops.length) { const op = ops[i]; const fn = ops[i + 1]; i += 2; if ((op === 'optionalAccess' || op === 'optionalCall') && value == null) { return undefined; } if (op === 'access' || op === 'optionalAccess') { lastAccessLHS = value; value = fn(value); } else if (op === 'call' || op === 'optionalCall') { value = fn((...args) => value.call(lastAccessLHS, ...args)); lastAccessLHS = undefined; } } return value; }var _biggystring = require('biggystring');









var _types = require('edge-core-js/types');
var _ethers = require('ethers');






var _swapHelpers = require('../../../../util/swapHelpers');
var _utils = require('../../../../util/utils');

var _VELODROME_V1_ROUTER_ABI = require('../../abi/VELODROME_V1_ROUTER_ABI'); var _VELODROME_V1_ROUTER_ABI2 = _interopRequireDefault(_VELODROME_V1_ROUTER_ABI);
var _VELODROME_V2_ROUTER_ABI = require('../../abi/VELODROME_V2_ROUTER_ABI'); var _VELODROME_V2_ROUTER_ABI2 = _interopRequireDefault(_VELODROME_V2_ROUTER_ABI);
var _WRAPPED_OPTIMISM_ETH_ABI = require('../../abi/WRAPPED_OPTIMISM_ETH_ABI'); var _WRAPPED_OPTIMISM_ETH_ABI2 = _interopRequireDefault(_WRAPPED_OPTIMISM_ETH_ABI);
var _defiUtils = require('../../defiUtils');
var _uniV2Utils = require('../uniV2Utils');

const swapInfo = {
  pluginId: 'velodrome',
  isDex: true,
  displayName: 'Velodrome',
  supportEmail: 'support@edge.app'
}

const EXPIRATION_MS = 1000 * 60
const SLIPPAGE = '0.05'
const OPTIMISM_RPC = 'https://rpc.ankr.com/optimism/'
const WETH_TOKEN_ADDRESS = '0x4200000000000000000000000000000000000006'
const VELODROME_V1_ROUTER_ADDRESS = '0x9c12939390052919aF3155f41Bf4160Fd3666A6f'
const VELODROME_V2_ROUTER_ADDRESS = '0xa062aE8A9c5e11aaA026fc2670B0D65cCc8B2858'
const VELODROME_V2_FACTORY_ADDRESS =
  '0xF1046053aa5682b4F9a81b5481394DA16BE5FF5a'






 function makeVelodromePlugin(
  opts
) {
  const fetchSwapQuoteInner = async (
    request,
    uid
  ) => {
    const { fromWallet, toWallet, fromTokenId, toTokenId } = request

    if (
      // Velodrome does not support reverse quotes
      request.quoteFor === 'to' ||
      // Velodrome only supports Optimism
      fromWallet.currencyInfo.pluginId !== 'optimism' ||
      toWallet.currencyInfo.pluginId !== 'optimism'
    ) {
      throw new (0, _types.SwapCurrencyError)(swapInfo, request)
    }

    // Parse input/output token addresses. If either from or to swap sources
    // are for the native currency, convert the address to the wrapped equivalent.
    const inOutAddresses = _defiUtils.getInOutTokenAddresses.call(void 0, 
      fromWallet.currencyConfig,
      WETH_TOKEN_ADDRESS,
      fromTokenId,
      toTokenId
    )
    const { fromTokenAddress, toTokenAddress, isWrappingSwap } = inOutAddresses

    const provider = new _ethers.ethers.providers.JsonRpcProvider(OPTIMISM_RPC)
    // Calculate swap amounts
    const velodromeRouterV1 = new _ethers.ethers.Contract(
      VELODROME_V1_ROUTER_ADDRESS,
      _VELODROME_V1_ROUTER_ABI2.default,
      provider
    )
    const velodromeRouterV2 = new _ethers.ethers.Contract(
      VELODROME_V2_ROUTER_ADDRESS,
      _VELODROME_V2_ROUTER_ABI2.default,
      provider
    )

    const configs = [
      // V1
      {
        path: [[fromTokenAddress, toTokenAddress, true]],
        router: velodromeRouterV1
      },
      {
        path: [[fromTokenAddress, toTokenAddress, false]],
        router: velodromeRouterV1
      },
      // V2
      {
        path: [
          [fromTokenAddress, toTokenAddress, true, VELODROME_V2_FACTORY_ADDRESS]
        ],
        router: velodromeRouterV2
      },
      {
        path: [
          [
            fromTokenAddress,
            toTokenAddress,
            false,
            VELODROME_V2_FACTORY_ADDRESS
          ]
        ],
        router: velodromeRouterV2
      }
    ]

    // Try stable and volatile pools form both routers and choose the best one.
    const getAmounts = async (
      config
    




) => {
      try {
        const amounts = await _uniV2Utils.getSwapAmounts.call(void 0, 
          config.router,
          request,
          swapInfo,
          config.path,
          isWrappingSwap
        )
        return { ...config, ...amounts }
      } catch (e) {
        return {
          ...config,
          amountToSwap: '0',
          expectedAmountOut: '0'
        }
      }
    }

    const amounts = await Promise.all(
      configs.map(async config => await getAmounts(config))
    )
    const best = amounts.sort((a, b) =>
      _biggystring.gt.call(void 0, a.expectedAmountOut, b.expectedAmountOut) ? -1 : 1
    )[0]

    const { amountToSwap, expectedAmountOut, router, path } = best
    if (expectedAmountOut === '0') {
      throw new (0, _types.SwapCurrencyError)(swapInfo, request)
    }

    // Generate swap transactions
    const toAddress = (await toWallet.getReceiveAddress({ tokenId: null }))
      .publicAddress
    const expirationDate = new Date(Date.now() + EXPIRATION_MS)
    const deadline = Math.round(expirationDate.getTime() / 1000) // unix timestamp
    const customNetworkFee = _swapHelpers.customFeeCache.getFees(uid)
    const wrappedEthContract = new _ethers.ethers.Contract(
      WETH_TOKEN_ADDRESS,
      _WRAPPED_OPTIMISM_ETH_ABI2.default,
      provider
    )
    const swapTxs = await _uniV2Utils.getSwapTransactions.call(void 0, 
      provider,
      inOutAddresses,
      path,
      router,
      wrappedEthContract,
      amountToSwap,
      expectedAmountOut,
      toAddress,
      SLIPPAGE,
      deadline,
      _optionalChain([customNetworkFee, 'optionalAccess', _ => _.gasPrice])
    )
    const fromAddress = (await fromWallet.getReceiveAddress({ tokenId: null }))
      .publicAddress
    // toEdgeUnsignedTxs
    const edgeSpendInfos = swapTxs.map((swapTx, i) => {
      // Convert to our spendInfo
      const edgeSpendInfo = {
        tokenId: request.fromTokenId,
        spendTargets: [
          {
            nativeAmount:
              swapTxs.length === 2 && i === 0
                ? '0' // approval transactions don't have a value
                : amountToSwap,
            publicAddress: swapTx.to
          }
        ],
        memos:
          swapTx.data != null
            ? [{ type: 'hex', value: swapTx.data.replace(/^0x/, '') }]
            : undefined,
        customNetworkFee: {
          gasPrice:
            swapTx.gasPrice != null
              ? _ethers.ethers.utils.formatUnits(swapTx.gasPrice, 'gwei').toString()
              : '0',
          gasLimit: _nullishCoalesce(_optionalChain([swapTx, 'access', _2 => _2.gasLimit, 'optionalAccess', _3 => _3.toString, 'call', _4 => _4()]), () => ( '0'))
        },
        networkFeeOption: 'custom',
        assetAction: {
          assetActionType: 'swap'
        },
        savedAction: {
          actionType: 'swap',
          swapInfo,
          isEstimate: false,
          toAsset: {
            pluginId: request.toWallet.currencyInfo.pluginId,
            tokenId: request.toTokenId,
            nativeAmount: expectedAmountOut.toString()
          },
          fromAsset: {
            pluginId: request.fromWallet.currencyInfo.pluginId,
            tokenId: request.fromTokenId,
            nativeAmount: amountToSwap
          },
          payoutAddress: toAddress,
          payoutWalletId: request.toWallet.id,
          refundAddress: fromAddress
        }
      }

      return edgeSpendInfo
    })

    let spendInfo = edgeSpendInfos[0]
    let preTx
    if (edgeSpendInfos.length > 1) {
      spendInfo = edgeSpendInfos[1]
      const approvalSpendInfo = {
        ...edgeSpendInfos[0],
        assetAction: {
          assetActionType: 'tokenApproval'
        },
        savedAction: {
          actionType: 'tokenApproval',
          tokenApproved: {
            pluginId: fromWallet.currencyInfo.pluginId,
            tokenId: fromTokenId,
            nativeAmount: amountToSwap
          },
          tokenContractAddress: inOutAddresses.fromTokenAddress,
          contractAddress: router.address
        }
      }

      preTx = await request.fromWallet.makeSpend(approvalSpendInfo)
    }

    spendInfo = {
      ...spendInfo,
      assetAction: {
        assetActionType: 'swap'
      },
      savedAction: {
        actionType: 'swap',
        swapInfo,
        isEstimate: false,
        toAsset: {
          pluginId: request.toWallet.currencyInfo.pluginId,
          tokenId: request.toTokenId,
          nativeAmount: expectedAmountOut.toString()
        },
        fromAsset: {
          pluginId: request.fromWallet.currencyInfo.pluginId,
          tokenId: request.fromTokenId,
          nativeAmount: amountToSwap
        },
        payoutAddress: toAddress,
        payoutWalletId: request.toWallet.id,
        refundAddress: fromAddress
      }
    }
    _swapHelpers.customFeeCache.setFees(uid, spendInfo.customNetworkFee)

    return {
      request,
      spendInfo,
      swapInfo,
      fromNativeAmount: amountToSwap,
      expirationDate,
      preTx
    }
  }

  const out = {
    swapInfo,
    async fetchSwapQuote(req) {
      const request = _utils.convertRequest.call(void 0, req)

      const uid = _swapHelpers.customFeeCache.createUid()

      const newRequest = await _swapHelpers.getMaxSwappable.call(void 0, 
        fetchSwapQuoteInner,
        request,
        uid
      )
      const swapOrder = await fetchSwapQuoteInner(newRequest, uid)
      return await _swapHelpers.makeSwapPluginQuote.call(void 0, swapOrder)
    }
  }

  return out
} exports.makeVelodromePlugin = makeVelodromePlugin;
